import 'package:dartils/dartils.dart';
import 'package:test/test.dart';

import '_test_classes/test_object.dart';

void main() {
  group('removeDuplicates', () {
    test('should return a list of unique number values in the correct order',
        () {
      const duplicatedItems = [0, 5, 3, 3, 2, 6, 0, 2, 8, 3];

      List result = ListUtils.removeDuplicates(items: duplicatedItems);

      expect(result, [0, 5, 3, 2, 6, 8]);
    });

    test('should return a list of unique string values in the correct order',
        () {
      const duplicatedItems = [
        "Berlin",
        "Genf",
        "Paris",
        "Washington",
        "Berlin",
        "London",
        "Genf",
        "Genf",
        "New York",
        "NewYork"
      ];

      List result = ListUtils.removeDuplicates(items: duplicatedItems);

      expect(result, [
        "Berlin",
        "Genf",
        "Paris",
        "Washington",
        "London",
        "New York",
        "NewYork"
      ]);
    });

    test('should return an empty list if the given list is null', () {
      expect(ListUtils.removeDuplicates(items: null), []);
    });
  });

  group('removeDuplicatesBy', () {
    test(
        'should return a list of unique object identified by the given parameter "by"',
        () {
      TestObject peter = TestObject(name: 'Peter', age: 21, height: 1.87);
      TestObject harry = TestObject(name: 'Harry', age: 21, height: 1.80);
      TestObject max = TestObject(name: 'Max', age: 22, height: 1.85);
      List<TestObject> duplicatedItems = [peter, harry, max];

      List<TestObject> result = ListUtils.removeDuplicatesBy(
        items: duplicatedItems,
        by: (TestObject a) => a.age,
      );

      expect(result, [peter, max]);
    });

    test('should return an empty list if the given list is null', () {
      expect(ListUtils.removeDuplicatesBy(items: null, by: (int a) => a), []);
    });
  });

  group('emptyIfNull', () {
    test('should return the given list if the list is not null', () {
      const items = [1, 2, 3];

      List<int> result = ListUtils.emptyIfNull(items: items);

      expect(result, items);
    });

    test('should return an empty list if the given list is null', () {
      List<int> result = ListUtils.emptyIfNull(items: null);

      expect(result, []);
    });
  });

  group('containsDuplicates', () {
    test(
        'should return true if the given list contains at least one duplicated value',
        () {
      const duplicatedItems = [0, 5, 3, 3, 2, 6, 0, 2, 8, 3];

      bool result = ListUtils.containsDuplicates(items: duplicatedItems);

      expect(result, true);
    });

    test('should return false if the given list contains no duplicated values',
        () {
      const uniqueItems = [0, 5, 3, 2, 6, 8];

      bool result = ListUtils.containsDuplicates(items: uniqueItems);

      expect(result, false);
    });

    test('should return false if the given list is null', () {
      expect(ListUtils.containsDuplicates(items: null), false);
    });
  });

  group('sortByNumber', () {
    test('should sort the given list by the given number in ascending order',
        () {
      TestObject peter = TestObject(name: 'Peter', age: 21, height: 1.87);
      TestObject harry = TestObject(name: 'Harry', age: 21, height: 1.80);
      TestObject max = TestObject(name: 'Max', age: 22, height: 1.85);
      List<TestObject> duplicatedItems = [peter, harry, max];

      ListUtils.sortByNumber(
        items: duplicatedItems,
        by: (TestObject a) => a.height,
      );

      expect(duplicatedItems, [harry, max, peter]);
    });

    test('should sort the given list by the given number in descending order',
        () {
      TestObject peter = TestObject(name: 'Peter', age: 21, height: 1.87);
      TestObject harry = TestObject(name: 'Harry', age: 21, height: 1.80);
      TestObject max = TestObject(name: 'Max', age: 22, height: 1.85);
      List<TestObject> duplicatedItems = [peter, harry, max];

      ListUtils.sortByNumber(
        items: duplicatedItems,
        by: (TestObject a) => a.height,
        descending: true,
      );

      expect(duplicatedItems, [peter, max, harry]);
    });
  });

  group('sortByString', () {
    test('should sort the given list by the given string in ascending order',
        () {
      TestObject peter = TestObject(name: 'Peter', age: 21, height: 1.87);
      TestObject harry = TestObject(name: 'Harry', age: 21, height: 1.80);
      TestObject max = TestObject(name: 'Max', age: 22, height: 1.85);
      List<TestObject> duplicatedItems = [peter, harry, max];

      ListUtils.sortByString(
        items: duplicatedItems,
        by: (TestObject a) => a.name,
      );

      expect(duplicatedItems, [max, harry, peter]);
    });

    test('should sort the given list by the given string in descending order',
        () {
      TestObject peter = TestObject(name: 'Peter', age: 21, height: 1.87);
      TestObject harry = TestObject(name: 'Harry', age: 21, height: 1.80);
      TestObject max = TestObject(name: 'Max', age: 22, height: 1.85);
      List<TestObject> duplicatedItems = [peter, harry, max];

      ListUtils.sortByString(
        items: duplicatedItems,
        by: (TestObject a) => a.name,
        descending: true,
      );

      expect(duplicatedItems, [peter, harry, max]);
    });
  });
}
